/*
 * Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * Copyright (c) 2019-2020 Mellanox Technologies LTD. ALL RIGHTS RESERVED.
 * See file LICENSE for terms.
 */

#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_CALCULATE_PORT_GROUPS_TASK_CPP
#include "osm_calculate_port_groups_task.hpp"
#include "osm_parallel_port_groups_calculator.hpp"
#include "osm_armgr.hpp"
#include "osm_ar_algorithm.hpp"

void
OSMCalculatePortGroupsTreeTask::Run()
{
	m_port_groups_calculator_->GetAdaptiveRoutingManager()->
		ARCalculateSwitchPortGroupsTree(
			*m_sw_db_entry_,
			m_port_groups_calculator_->GetTreeAlgorithmContext());

	m_port_groups_calculator_->OnTaskEnd();
}

void
OSMCalculatePortGroupsTask::Run()
{
	switch (m_port_groups_calculator_->GetSubTask()) {
		case OSMParallelPortGroupsCalculator::SUB_TASK_CALCULATE_PORT_GROUPS:
		{
			m_ar_algorithm_->CalculateSwitchPortGroups(
				*m_sw_db_entry_,
				m_port_groups_calculator_->GetLidMapping(),
				m_dst_sw_lid_to_connection_, /* temp data buffer used for calculation */
				m_base_lid_to_port_);       /* temp data buffer used for calculation */
			break;
		}
		case OSMParallelPortGroupsCalculator::SUB_TASK_CALCULATE_STATIC_ROUTE_INFO:
		{
			m_ar_algorithm_->BuildStaticRouteInfoToSwitch(
				*m_sw_db_entry_, false);       /* first calculation */
			break;
		}
		case OSMParallelPortGroupsCalculator::SUB_TASK_RECALCULATE_STATIC_ROUTE_INFO:
		{
			m_ar_algorithm_->BuildStaticRouteInfoToSwitch(
				*m_sw_db_entry_, true);       /* rebuild_route_info */
			break;
		}
		default:
			OSM_AR_LOG(m_ar_algorithm_->GetLog(), OSM_LOG_ERROR,
				   "ERR 9601 Unsupported sub task\n");
	}

	m_port_groups_calculator_->OnTaskEnd();
}
