%{
/*
 * Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stddef.h>
#include <complib/cl_hashmap.h>
#include <opensm/osm_opensm.h>
#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_TENANT_PARSER_Y_Y
#include <opensm/osm_tenant_mgr.h>

/* -------------------------------------------------------------------------------------------------
 *
 * TENANT interface functions.
 */

static void tenant_map_init(osm_tenant_parser_t *p_parser, char *id) {
	p_parser->map_id = strtoull(id, NULL, 0);
}

static void tenant_start_physical_list(osm_tenant_parser_t *p_parser) {
	p_parser->p_list = &p_parser->p_tenant->physical_guids;
}

static void tenant_start_virtual_list(osm_tenant_parser_t *p_parser) {
	p_parser->p_list = &p_parser->p_tenant->virtual_guids;
}

static void tenant_update_list(osm_tenant_parser_t *p_parser, char *id) {
	uint64_t	guid = (uint64_t)strtoul(id, NULL, 0);

	osm_tenant_guidlist_insert_guid(p_parser->p_list, guid, p_parser->map_id);
}

static void tenant_end_list(osm_tenant_parser_t *p_parser) {
	p_parser->p_list = NULL;
}

/* -------------------------------------------------------------------------------------------------
 *
 * YACC interface functions.
 */

#define YYSTYPE char *

extern int __tenant_parser_get_lineno(void);

char *__tenant_parser_yylval = NULL;

int __tenant_parser_lex();

static void __tenant_parser_error(osm_tenant_parser_t *p_parser, const char *reason)
{
	OSM_LOG(p_parser->p_log, OSM_LOG_ERROR, "ERR AE07: parsing error near line %d\n", __tenant_parser_get_lineno());
}

%}

%parse-param { osm_tenant_parser_t *p_parser }

%start tenant_file

%token COMMENT
%token COLON
%token MAP_GROUP
%token NAME
%token END_MAP_GROUP
%token IDENTIFIER
%token GUID
%token PHYSICAL_PORT_GUID
%token VIRTUAL_PORT_GUID
%token COMMA
%token NUMBER
%token ERROR

%%

tenant_file	: map_groups
		| /* empty file */
		;

map_groups	: map_groups map_group
		| map_group
		;

map_group	: MAP_GROUP NAME COLON map_name map_list END_MAP_GROUP
		;

map_name	: NUMBER { tenant_map_init(p_parser, __tenant_parser_yylval); }
		;

map_list	: map_list map_entry
		| map_entry
		;

map_entry	: guid_type COLON guid_list { tenant_end_list(p_parser); }
		;

guid_type	: VIRTUAL_PORT_GUID  { tenant_start_virtual_list(p_parser); }
		| PHYSICAL_PORT_GUID { tenant_start_physical_list(p_parser); }
		;

guid_list	: guid_list COMMA guid
		| guid
		;

guid		: NUMBER { tenant_update_list(p_parser, __tenant_parser_yylval); }
		|
		;

